#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#
#
# This file is part of Mapnik (c++ mapping toolkit)
# Copyright (C) 2005 Jean-Francois Doyon
#
# Mapnik is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
from __future__ import print_function
import sys
from os import path
import mapnik

# Instanciate a map, giving it a width and height. Remember: the word "map" is
# reserved in Python! :)

root = path.dirname(__file__)
m = mapnik.Map(800,600,"+proj=merc +a=6378137 +b=6378137 +lat_ts=0.0 +lon_0=0.0 +x_0=0.0 +y_0=0 +k=1.0 +units=m +nadgrids=@null +no_defs")

# Set its background colour. More on colours later ...

m.background = mapnik.Color('white')

# Now we can start adding layers, in stacking order (i.e. bottom layer first)

# Canadian Provinces (Polygons)

# Instanciate a layer.  The parameters depend on the type of data:
# shape:
#     type='shape'
#     file='/path/to/shape'
# raster:
#     type='raster'
#     file='/path/to/raster'
# postgis:
#     type='postgis'
#     host='127.0.0.1'
#     dbname='mydatabase'
#     user='myusername'
#     password='mypassword'
#     table= TODO

provpoly_lyr = mapnik.Layer('Provinces')
provpoly_lyr.srs = "+proj=lcc +ellps=GRS80 +lat_0=49 +lon_0=-95 +lat+1=49 +lat_2=77 +datum=NAD83 +units=m +no_defs"
provpoly_lyr.datasource = mapnik.Shapefile(file=path.join(root,'../data/boundaries'), encoding='latin1')

# We then define a style for the layer.  A layer can have one or many styles.
# Styles are named, so they can be shared across different layers.
# Multiple styles per layer behaves functionally like multiple layers.  The
# data is completely re-scanned for each style within one layer, and a style
# will be drawn entirely "above" the previous one.  Performance wise using
# multiple styles in one layer is the same has having multiple layers.
# The paradigm is useful mostly as a convenience.

provpoly_style = mapnik.Style()

# A Style needs one or more rules.  A rule will normally consist of a filter
# for feature selection, and one or more symbolizers.

provpoly_rule_on = mapnik.Rule()

# A Expression() allows the selection of features to which the symbology will
# be applied.  More on Mapnik expressions can be found in Tutorial #2.
# A given feature can only match one filter per rule per style.

provpoly_rule_on.filter = mapnik.Expression("[NAME_EN] = 'Ontario'")

# Here a symbolizer is defined.  Available are:
#     - LineSymbolizer(Color(),<width>)
#     - LineSymbolizer(Stroke())
#     - PolygonSymbolizer(Color())
#     - PointSymbolizer(<file>,<type>,<width>,<height>)

# Some of them can accept a Color() instance, which can be created with:
#     - Color(<red>, <green>, <blue>)
#     - Color(<red>, <green>, <blue>, <alpha>)
#     - Color(<string>) where <string> will be something like '#00FF00'
#       or '#0f0' or 'green'

sym = mapnik.PolygonSymbolizer()
sym.fill = mapnik.Color(250, 190, 183);
provpoly_rule_on.symbols.append(sym)
provpoly_style.rules.append(provpoly_rule_on)

provpoly_rule_qc = mapnik.Rule()
provpoly_rule_qc.filter = mapnik.Expression("[NOM_FR] = 'Québec'")
sym = mapnik.PolygonSymbolizer()
sym.fill = mapnik.Color(217, 235, 203)
provpoly_rule_qc.symbols.append(sym)
provpoly_style.rules.append(provpoly_rule_qc)

# Add the style to the map, giving it a name.  This is the name that will be
# used to refer to it from here on.  Having named styles allows them to be
# re-used throughout the map.

m.append_style('provinces', provpoly_style)

# Then associate the style to the layer itself.

provpoly_lyr.styles.append('provinces')

# Then add the layer to the map.  In reality, it's the order in which you
# append them to the map that will determine the drawing order, though by
# convention it is recommended to define them in drawing order as well.

m.layers.append(provpoly_lyr)

# Drainage

# A simple example ...

qcdrain_lyr = mapnik.Layer('Quebec Hydrography')
qcdrain_lyr.srs = "+proj=lcc +ellps=GRS80 +lat_0=49 +lon_0=-95 +lat+1=49 +lat_2=77 +datum=NAD83 +units=m +no_defs"
qcdrain_lyr.datasource = mapnik.Shapefile(file=path.join(root,'../data/qcdrainage'))

qcdrain_style = mapnik.Style()
qcdrain_rule = mapnik.Rule()
qcdrain_rule.filter = mapnik.Expression('[HYC] = 8')
sym = mapnik.PolygonSymbolizer()
sym.fill = mapnik.Color(153, 204, 255)
sym.smooth = 1.0 # very smooth
qcdrain_rule.symbols.append(sym)
qcdrain_style.rules.append(qcdrain_rule)

m.append_style('drainage', qcdrain_style)
qcdrain_lyr.styles.append('drainage')
m.layers.append(qcdrain_lyr)

# In this case, we have 2 data sets with similar schemas (same filtering
# attributes, and same desired style), so we're going to
# re-use the style defined in the above layer for the next one.

ondrain_lyr = mapnik.Layer('Ontario Hydrography')
ondrain_lyr.srs = "+proj=lcc +ellps=GRS80 +lat_0=49 +lon_0=-95 +lat+1=49 +lat_2=77 +datum=NAD83 +units=m +no_defs"
ondrain_lyr.datasource = mapnik.Shapefile(file=path.join(root,'../data/ontdrainage'))

ondrain_lyr.styles.append('drainage')
m.layers.append(ondrain_lyr)

# Provincial boundaries

provlines_lyr = mapnik.Layer('Provincial borders')
provlines_lyr.srs = "+proj=lcc +ellps=GRS80 +lat_0=49 +lon_0=-95 +lat+1=49 +lat_2=77 +datum=NAD83 +units=m +no_defs"
provlines_lyr.datasource = mapnik.Shapefile(file=path.join(root,'../data/boundaries_l'))

# Here we define a "dash dot dot dash" pattern for the provincial boundaries.

provlines_style = mapnik.Style()
provlines_rule = mapnik.Rule()
sym = mapnik.LineSymbolizer()
# FIXME - currently adding dash arrays is broken
# https://github.com/mapnik/mapnik/issues/2324
sym.stroke = mapnik.Color('black')
sym.stroke_width = 1
provlines_rule.symbols.append(sym)
provlines_style.rules.append(provlines_rule)

m.append_style('provlines', provlines_style)
provlines_lyr.styles.append('provlines')
m.layers.append(provlines_lyr)

# Roads 3 and 4 (The "grey" roads)

roads34_lyr = mapnik.Layer('Roads')
roads34_lyr.srs = "+proj=lcc +ellps=GRS80 +lat_0=49 +lon_0=-95 +lat+1=49 +lat_2=77 +datum=NAD83 +units=m +no_defs"
# create roads datasource (we're going to re-use it later)

roads34_lyr.datasource = mapnik.Shapefile(file=path.join(root,'../data/roads'))

roads34_style = mapnik.Style()
roads34_rule = mapnik.Rule()
roads34_rule.filter = mapnik.Expression('([CLASS] = 3) or ([CLASS] = 4)')

# With lines of a certain width, you can control how the ends
# are closed off using line_cap as below.

# Available options are:
# line_cap: BUTT_CAP, SQUARE_CAP, ROUND_CAP
# line_join: MITER_JOIN, MITER_REVERT_JOIN, ROUND_JOIN, BEVEL_JOIN

# And one last Stroke() attribute not used here is "opacity", which
# can be set to a numerical value.

sym = mapnik.LineSymbolizer()
sym.stroke = mapnik.Color(171,158,137)
sym.stroke_width = 2
sym.stroke_linecap = mapnik.stroke_linecap.ROUND_CAP

roads34_rule.symbols.append(sym)
roads34_style.rules.append(roads34_rule)

m.append_style('smallroads', roads34_style)
roads34_lyr.styles.append('smallroads')
m.layers.append(roads34_lyr)

# Roads 2 (The thin yellow ones)

roads2_lyr = mapnik.Layer('Roads')
roads2_lyr.srs = "+proj=lcc +ellps=GRS80 +lat_0=49 +lon_0=-95 +lat+1=49 +lat_2=77 +datum=NAD83 +units=m +no_defs"
# Just get a copy from roads34_lyr
roads2_lyr.datasource = roads34_lyr.datasource

roads2_style_1 = mapnik.Style()
roads2_rule_1 = mapnik.Rule()
roads2_rule_1.filter = mapnik.Expression('[CLASS] = 2')

sym = mapnik.LineSymbolizer()
sym.stroke = mapnik.Color(171,158,137)
sym.stroke_width = 4
sym.stroke_linecap = mapnik.stroke_linecap.ROUND_CAP
roads2_rule_1.symbols.append(sym)
roads2_style_1.rules.append(roads2_rule_1)

m.append_style('road-border', roads2_style_1)

roads2_style_2 = mapnik.Style()
roads2_rule_2 = mapnik.Rule()
roads2_rule_2.filter = mapnik.Expression('[CLASS] = 2')
sym = mapnik.LineSymbolizer()
sym.stroke = mapnik.Color(255,250,115)
sym.stroke_linecap = mapnik.stroke_linecap.ROUND_CAP
sym.stroke_width = 2
roads2_rule_2.symbols.append(sym)
roads2_style_2.rules.append(roads2_rule_2)

m.append_style('road-fill', roads2_style_2)

roads2_lyr.styles.append('road-border')
roads2_lyr.styles.append('road-fill')

m.layers.append(roads2_lyr)

# Roads 1 (The big orange ones, the highways)

roads1_lyr = mapnik.Layer('Roads')
roads1_lyr.srs = "+proj=lcc +ellps=GRS80 +lat_0=49 +lon_0=-95 +lat+1=49 +lat_2=77 +datum=NAD83 +units=m +no_defs"
roads1_lyr.datasource = roads34_lyr.datasource

roads1_style_1 = mapnik.Style()
roads1_rule_1 = mapnik.Rule()
roads1_rule_1.filter = mapnik.Expression('[CLASS] = 1')
sym = mapnik.LineSymbolizer()
sym.stroke = mapnik.Color(188,149,28)
sym.stroke_linecap = mapnik.stroke_linecap.ROUND_CAP
sym.stroke_width = 7
roads1_rule_1.symbols.append(sym)
roads1_style_1.rules.append(roads1_rule_1)
m.append_style('highway-border', roads1_style_1)

roads1_style_2 = mapnik.Style()
roads1_rule_2 = mapnik.Rule()
roads1_rule_2.filter = mapnik.Expression('[CLASS] = 1')
sym.stroke = mapnik.Color(242,191,36)
sym.stroke_linecap = mapnik.stroke_linecap.ROUND_CAP
sym.stroke_width = 5
roads1_rule_2.symbols.append(sym)
roads1_style_2.rules.append(roads1_rule_2)

m.append_style('highway-fill', roads1_style_2)

roads1_lyr.styles.append('highway-border')
roads1_lyr.styles.append('highway-fill')

m.layers.append(roads1_lyr)

# Populated Places

popplaces_lyr = mapnik.Layer('Populated Places')
popplaces_lyr.srs = "+proj=lcc +ellps=GRS80 +lat_0=49 +lon_0=-95 +lat+1=49 +lat_2=77 +datum=NAD83 +units=m +no_defs"
popplaces_lyr.datasource = mapnik.Shapefile(file=path.join(root,'../data/popplaces'),encoding='latin1')

popplaces_style = mapnik.Style()
popplaces_rule = mapnik.Rule()

# And here we have a TextSymbolizer, used for labeling.
# The first parameter is the name of the attribute to use as the source of the
# text to label with.  Then there is font size in points (I think?), and colour.

# TODO - currently broken: https://github.com/mapnik/mapnik/issues/2324
#popplaces_text_symbolizer = mapnik.TextSymbolizer(mapnik.Expression("[GEONAME]"),
#                                           'DejaVu Sans Book',
#                                           10, mapnik.Color('black'))

# We set a "halo" around the text, which looks like an outline if thin enough,
# or an outright background if large enough.
#popplaces_text_symbolizer.label_placement= mapnik.label_placement.POINT_PLACEMENT
#popplaces_text_symbolizer.halo_fill = mapnik.Color(255,255,200)
#popplaces_text_symbolizer.halo_radius = 1
#popplaces_text_symbolizer.avoid_edges = True
#popplaces_text_symbolizer.minimum_padding = 30
#popplaces_rule.symbols.append(popplaces_text_symbolizer)

popplaces_style.rules.append(popplaces_rule)

m.append_style('popplaces', popplaces_style)
popplaces_lyr.styles.append('popplaces')
m.layers.append(popplaces_lyr)

# Draw map

# Set the initial extent of the map in 'master' spherical Mercator projection
m.zoom_to_box(mapnik.Box2d(-8024477.28459,5445190.38849,-7381388.20071,5662941.44855))

# Render map
im = mapnik.Image(m.width,m.height)
mapnik.render(m, im)

# Save image to files
images_ = []
if mapnik.has_png():
    im.save('demo.png', 'png') # true-colour RGBA
    images_.append('demo.png')

    # old behavior, now can do 'png8:c=256'
    im.save('demo256.png', 'png256') # save to palette based (max 256 colours) png
    images_.append('demo256.png')

    im.save('demo64_binary_transparency.png', 'png8:c=64:t=1')
    images_.append('demo64_binary_transparency.png')

    im.save('demo128_colors_hextree_no_alpha.png', 'png8:c=100:m=h:t=0')
    images_.append('demo128_colors_hextree_no_alpha.png')

if mapnik.has_jpeg():
    im.save('demo_high.jpg', 'jpeg100')
    images_.append('demo_high.jpg')

    im.save('demo_low.jpg', 'jpeg50')
    images_.append('demo_low.jpg')

if mapnik.has_tiff():
    im.save('demo.tif', 'tiff')
    images_.append('demo.tif')

if mapnik.has_webp():
    im.save('demo.webp', 'webp') # default quality is 90
    images_.append('demo.webp')

    im.save('demo_highest.webp', 'webp:quality=100')
    images_.append('demo_med.webp')

    im.save('demo_low.webp', 'webp:quality=50')
    images_.append('demo_low.webp')


# Render cairo examples
if  mapnik.has_cairo():
    mapnik.render_to_file(m,'demo.pdf')
    images_.append('demo.pdf')
    mapnik.render_to_file(m,'demo.ps')
    images_.append('demo.ps')
    mapnik.render_to_file(m,'demo.svg')
    images_.append('demo.svg')
    mapnik.render_to_file(m,'demo_cairo_rgb24.png','RGB24')
    images_.append('demo_cairo_rgb.png')
    mapnik.render_to_file(m,'demo_cairo_argb32.png','ARGB32')
    images_.append('demo_cairo_argb.png')

print ("\n\n", len(images_), "maps have been rendered in the current directory:")

for im_ in images_:
    print ("-", im_)

print ("\n\nHave a look!\n\n")

mapnik.save_map(m,"map.xml")



#! /usr/bin/env python

import os
import os.path
import re
import shutil
import subprocess
import sys
import glob
from distutils import sysconfig
from ctypes.util import find_library

from setuptools import Command, Extension, setup

PYTHON3 = sys.version_info.major == 3


# Utils
def check_output(args):
    output = subprocess.check_output(args)
    if PYTHON3:
        # check_output returns bytes in PYTHON3.
        output = output.decode()
    return output.rstrip('\n')


def clean_boost_name(name):
    name = name.split('.')[0]
    if name.startswith('lib'):
        name = name[3:]
    return name


def find_boost_library(_id):
    suffixes = [
        "",  # standard naming
        "-mt"  # former naming schema for multithreading build
    ]
    if "python" in _id:
        # Debian naming convention for versions installed in parallel
        suffixes.insert(0, "-py%d%d" % (sys.version_info.major,
                                        sys.version_info.minor))
        # standard suffix for Python3
        suffixes.insert(1, sys.version_info.major)
    for suf in suffixes:
        name = "%s%s" % (_id, suf)
        lib = find_library(name)
        if lib is not None:
            return name


def get_boost_library_names():
    wanted = ['boost_python', 'boost_thread', 'boost_system']
    found = []
    missing = []
    for _id in wanted:
        name = os.environ.get("%s_LIB" % _id.upper(), find_boost_library(_id))
        if name:
            found.append(name)
        else:
            missing.append(_id)
    if missing:
        msg = ""
        for name in missing:
            msg += ("\nMissing {} boost library, try to add its name with "
                    "{}_LIB environment var.").format(name, name.upper())
        raise EnvironmentError(msg)
    return found


class WhichBoostCommand(Command):
    description = 'Output found boost names. Useful for debug.'
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        print("\n".join(get_boost_library_names()))


cflags = sysconfig.get_config_var('CFLAGS')
sysconfig._config_vars['CFLAGS'] = re.sub(
    ' +', ' ', cflags.replace('-g ', '').replace('-Os', '').replace('-arch i386', ''))
opt = sysconfig.get_config_var('OPT')
sysconfig._config_vars['OPT'] = re.sub(
    ' +', ' ', opt.replace('-g ', '').replace('-Os', ''))
ldshared = sysconfig.get_config_var('LDSHARED')
sysconfig._config_vars['LDSHARED'] = re.sub(
    ' +', ' ', ldshared.replace('-g ', '').replace('-Os', '').replace('-arch i386', ''))
ldflags = sysconfig.get_config_var('LDFLAGS')
sysconfig._config_vars['LDFLAGS'] = re.sub(
    ' +', ' ', ldflags.replace('-g ', '').replace('-Os', '').replace('-arch i386', ''))
pycflags = sysconfig.get_config_var('PY_CFLAGS')
sysconfig._config_vars['PY_CFLAGS'] = re.sub(
    ' +', ' ', pycflags.replace('-g ', '').replace('-Os', '').replace('-arch i386', ''))
sysconfig._config_vars['CFLAGSFORSHARED'] = ''
os.environ['ARCHFLAGS'] = ''

if os.environ.get("MASON_BUILD", "false") == "true":
    # run bootstrap.sh to get mason builds
    subprocess.call(['./bootstrap.sh'])
    mapnik_config = 'mason_packages/.link/bin/mapnik-config'
    mason_build = True
else:
    mapnik_config = 'mapnik-config'
    mason_build = False


linkflags = []
lib_path = os.path.join(check_output([mapnik_config, '--prefix']),'lib')
linkflags.extend(check_output([mapnik_config, '--libs']).split(' '))
linkflags.extend(check_output([mapnik_config, '--ldflags']).split(' '))
linkflags.extend(check_output([mapnik_config, '--dep-libs']).split(' '))
linkflags.extend([
'-lmapnik-wkt',
'-lmapnik-json',
] + ['-l%s' % i for i in get_boost_library_names()])

# Dynamically make the mapnik/paths.py file
f_paths = open('mapnik/paths.py', 'w')
f_paths.write('import os\n')
f_paths.write('\n')

input_plugin_path = check_output([mapnik_config, '--input-plugins'])
font_path = check_output([mapnik_config, '--fonts'])

if mason_build:
    try:
        if sys.platform == 'darwin':
            base_f = 'libmapnik.dylib'
        else:
            base_f = 'libmapnik.so'
        f = os.path.join(lib_path, base_f)
        if not os.path.exists(os.path.join('mapnik', 'lib')):
            os.makedirs(os.path.join('mapnik', 'lib'))
        shutil.copyfile(f, os.path.join('mapnik', 'lib', base_f))
    except shutil.Error:
        pass
    input_plugin_files = os.listdir(input_plugin_path)
    input_plugin_files = [os.path.join(
        input_plugin_path, f) for f in input_plugin_files]
    if not os.path.exists(os.path.join('mapnik', 'lib', 'mapnik', 'input')):
        os.makedirs(os.path.join('mapnik', 'lib', 'mapnik', 'input'))
    for f in input_plugin_files:
        try:
            shutil.copyfile(f, os.path.join(
                'mapnik', 'lib', 'mapnik', 'input', os.path.basename(f)))
        except shutil.Error:
            pass
    font_files = os.listdir(font_path)
    font_files = [os.path.join(font_path, f) for f in font_files]
    if not os.path.exists(os.path.join('mapnik', 'lib', 'mapnik', 'fonts')):
        os.makedirs(os.path.join('mapnik', 'lib', 'mapnik', 'fonts'))
    for f in font_files:
        try:
            shutil.copyfile(f, os.path.join(
                'mapnik', 'lib', 'mapnik', 'fonts', os.path.basename(f)))
        except shutil.Error:
            pass
    f_paths.write(
        'mapniklibpath = os.path.join(os.path.dirname(os.path.realpath(__file__)), "lib")\n')
    f_paths.write("inputpluginspath = os.path.join(mapniklibpath, 'mapnik', 'input')\n")
    f_paths.write("fontscollectionpath = os.path.join(mapniklibpath, 'mapnik', 'fonts')\n")
    f_paths.write(
        "__all__ = [mapniklibpath,inputpluginspath,fontscollectionpath]\n")
    f_paths.close()
else:
    if os.environ.get('LIB_DIR_NAME'):
        mapnik_lib_path = lib_path + os.environ.get('LIB_DIR_NAME')
    else:
        mapnik_lib_path = lib_path + "/mapnik"
    f_paths.write("mapniklibpath = '{path}'\n".format(path=mapnik_lib_path))
    f_paths.write('mapniklibpath = os.path.normpath(mapniklibpath)\n')
    f_paths.write(
        "inputpluginspath = '{path}'\n".format(path=input_plugin_path))
    f_paths.write(
        "fontscollectionpath = '{path}'\n".format(path=font_path))
    f_paths.write(
        "__all__ = [mapniklibpath,inputpluginspath,fontscollectionpath]\n")
    f_paths.close()


if mason_build:

    share_dir = 'share'

    for dep in ['icu','gdal','proj']:
        share_path = os.path.join('mapnik', share_dir, dep)
        if not os.path.exists(share_path):
            os.makedirs(share_path)

    icu_path = 'mason_packages/.link/share/icu/*/*.dat'
    icu_files = glob.glob(icu_path)
    if len(icu_files) != 1:
        raise Exception("Failed to find icu dat file at "+ icu_path)
    for f in icu_files:
        shutil.copyfile(f, os.path.join(
            'mapnik', share_dir, 'icu', os.path.basename(f)))

    gdal_path = 'mason_packages/.link/share/gdal/'
    gdal_files = os.listdir(gdal_path)
    gdal_files = [os.path.join(gdal_path, f) for f in gdal_files]
    for f in gdal_files:
        try:
            shutil.copyfile(f, os.path.join(
                'mapnik', share_dir, 'gdal', os.path.basename(f)))
        except shutil.Error:
            pass

    proj_path = 'mason_packages/.link/share/proj/'
    proj_files = os.listdir(proj_path)
    proj_files = [os.path.join(proj_path, f) for f in proj_files]
    for f in proj_files:
        try:
            shutil.copyfile(f, os.path.join(
                'mapnik', share_dir, 'proj', os.path.basename(f)))
        except shutil.Error:
            pass

extra_comp_args = check_output([mapnik_config, '--cflags']).split(' ')

extra_comp_args = list(filter(lambda arg: arg != "-fvisibility=hidden", extra_comp_args))

if os.environ.get("PYCAIRO", "false") == "true":
    try:
        extra_comp_args.append('-DHAVE_PYCAIRO')
        print("-I%s/include/pycairo".format(sys.exec_prefix))
        extra_comp_args.append("-I{0}/include/pycairo".format(sys.exec_prefix))
        #extra_comp_args.extend(check_output(["pkg-config", '--cflags', 'pycairo']).strip().split(' '))
        #linkflags.extend(check_output(["pkg-config", '--libs', 'pycairo']).strip().split(' '))
    except:
        raise Exception("Failed to find compiler options for pycairo")

if sys.platform == 'darwin':
    extra_comp_args.append('-mmacosx-version-min=10.11')
    # silence warning coming from boost python macros which
    # would is hard to silence via pragma
    extra_comp_args.append('-Wno-parentheses-equality')
    linkflags.append('-mmacosx-version-min=10.11')
else:
    linkflags.append('-lrt')
    linkflags.append('-Wl,-z,origin')
    linkflags.append('-Wl,-rpath=$ORIGIN/lib')

if os.environ.get("CC", False) == False:
    os.environ["CC"] = check_output([mapnik_config, '--cxx'])
if os.environ.get("CXX", False) == False:
    os.environ["CXX"] = check_output([mapnik_config, '--cxx'])

setup(
    name="mapnik",
    version="0.1",
    packages=['mapnik','mapnik.printing'],
    author="Blake Thompson",
    author_email="flippmoke@gmail.com",
    description="Python bindings for Mapnik",
    license="GNU LESSER GENERAL PUBLIC LICENSE",
    keywords="mapnik mapbox mapping cartography",
    url="http://mapnik.org/",
    tests_require=[
        'nose',
    ],
    package_data={
        'mapnik': ['lib/*.*', 'lib/*/*/*', 'share/*/*'],
    },
    test_suite='nose.collector',
    cmdclass={
        'whichboost': WhichBoostCommand,
    },
    ext_modules=[
        Extension('mapnik._mapnik', [
            'src/mapnik_color.cpp',
            'src/mapnik_coord.cpp',
            'src/mapnik_datasource.cpp',
            'src/mapnik_datasource_cache.cpp',
            'src/mapnik_envelope.cpp',
            'src/mapnik_expression.cpp',
            'src/mapnik_feature.cpp',
            'src/mapnik_featureset.cpp',
            'src/mapnik_font_engine.cpp',
            'src/mapnik_fontset.cpp',
            'src/mapnik_gamma_method.cpp',
            'src/mapnik_geometry.cpp',
            'src/mapnik_grid.cpp',
            'src/mapnik_grid_view.cpp',
            'src/mapnik_image.cpp',
            'src/mapnik_image_view.cpp',
            'src/mapnik_label_collision_detector.cpp',
            'src/mapnik_layer.cpp',
            'src/mapnik_logger.cpp',
            'src/mapnik_map.cpp',
            'src/mapnik_palette.cpp',
            'src/mapnik_parameters.cpp',
            'src/mapnik_proj_transform.cpp',
            'src/mapnik_projection.cpp',
            'src/mapnik_python.cpp',
            'src/mapnik_query.cpp',
            'src/mapnik_raster_colorizer.cpp',
            'src/mapnik_rule.cpp',
            'src/mapnik_scaling_method.cpp',
            'src/mapnik_style.cpp',
            'src/mapnik_svg_generator_grammar.cpp',
            'src/mapnik_symbolizer.cpp',
            'src/mapnik_view_transform.cpp',
            'src/python_grid_utils.cpp',
        ],
            language='c++',
            extra_compile_args=extra_comp_args,
            extra_link_args=linkflags,
        )
    ]
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import sys

from nose.tools import eq_, raises

import mapnik

from .utilities import run_all

PYTHON3 = sys.version_info[0] == 3
if PYTHON3:
    long = int
    unicode = str


if hasattr(mapnik, 'Expression'):
    mapnik.Filter = mapnik.Expression

map_ = '''<Map>
    <Style name="s">
        <Rule>
            <Filter><![CDATA[(([region]>=0) and ([region]<=50))]]></Filter>
        </Rule>
        <Rule>
            <Filter><![CDATA[([region]>=0) and ([region]<=50)]]></Filter>
        </Rule>
        <Rule>
            <Filter>

            <![CDATA[

            ([region] >= 0)

            and

            ([region] <= 50)
            ]]>

            </Filter>
        </Rule>
        <Rule>
            <Filter>([region]&gt;=0) and ([region]&lt;=50)</Filter>
        </Rule>
        <Rule>
            <Filter>
            ([region] &gt;= 0)
             and
            ([region] &lt;= 50)
            </Filter>
        </Rule>

    </Style>
    <Style name="s2" filter-mode="first">
        <Rule>
        </Rule>
        <Rule>
        </Rule>
    </Style>
</Map>'''


def test_filter_init():
    m = mapnik.Map(1, 1)
    mapnik.load_map_from_string(m, map_)
    filters = []
    filters.append(mapnik.Filter("([region]>=0) and ([region]<=50)"))
    filters.append(mapnik.Filter("(([region]>=0) and ([region]<=50))"))
    filters.append(mapnik.Filter("((([region]>=0) and ([region]<=50)))"))
    filters.append(mapnik.Filter('((([region]>=0) and ([region]<=50)))'))
    filters.append(mapnik.Filter('''((([region]>=0) and ([region]<=50)))'''))
    filters.append(mapnik.Filter('''
    ((([region]>=0)
    and
    ([region]<=50)))
    '''))
    filters.append(mapnik.Filter('''
    ([region]>=0)
    and
    ([region]<=50)
    '''))
    filters.append(mapnik.Filter('''
    ([region]
    >=
    0)
    and
    ([region]
    <=
    50)
    '''))

    s = m.find_style('s')

    for r in s.rules:
        filters.append(r.filter)

    first = filters[0]
    for f in filters:
        eq_(str(first), str(f))

    s = m.find_style('s2')

    eq_(s.filter_mode, mapnik.filter_mode.FIRST)


def test_geometry_type_eval():
    # clashing field called 'mapnik::geometry'
    context2 = mapnik.Context()
    context2.push('mapnik::geometry_type')
    f = mapnik.Feature(context2, 0)
    f["mapnik::geometry_type"] = 'sneaky'
    expr = mapnik.Expression("[mapnik::geometry_type]")
    eq_(expr.evaluate(f), 0)

    expr = mapnik.Expression("[mapnik::geometry_type]")
    context = mapnik.Context()

    # no geometry
    f = mapnik.Feature(context, 0)
    eq_(expr.evaluate(f), 0)
    eq_(mapnik.Expression("[mapnik::geometry_type]=0").evaluate(f), True)

    # POINT = 1
    f = mapnik.Feature(context, 0)
    f.geometry = mapnik.Geometry.from_wkt('POINT(10 40)')
    eq_(expr.evaluate(f), 1)
    eq_(mapnik.Expression("[mapnik::geometry_type]=point").evaluate(f), True)

    # LINESTRING = 2
    f = mapnik.Feature(context, 0)
    f.geometry = mapnik.Geometry.from_wkt('LINESTRING (30 10, 10 30, 40 40)')
    eq_(expr.evaluate(f), 2)
    eq_(mapnik.Expression(
        "[mapnik::geometry_type] = linestring").evaluate(f), True)

    # POLYGON = 3
    f = mapnik.Feature(context, 0)
    f.geometry = mapnik.Geometry.from_wkt(
        'POLYGON ((30 10, 10 20, 20 40, 40 40, 30 10))')
    eq_(expr.evaluate(f), 3)
    eq_(mapnik.Expression(
        "[mapnik::geometry_type] = polygon").evaluate(f), True)

    # COLLECTION = 4
    f = mapnik.Feature(context, 0)
    geom = mapnik.Geometry.from_wkt(
        'GEOMETRYCOLLECTION(POLYGON((1 1,2 1,2 2,1 2,1 1)),POINT(2 3),LINESTRING(2 3,3 4))')
    f.geometry = geom
    eq_(expr.evaluate(f), 4)
    eq_(mapnik.Expression(
        "[mapnik::geometry_type] = collection").evaluate(f), True)


def test_regex_match():
    context = mapnik.Context()
    context.push('name')
    f = mapnik.Feature(context, 0)
    f["name"] = 'test'
    expr = mapnik.Expression("[name].match('test')")
    eq_(expr.evaluate(f), True)  # 1 == True


def test_unicode_regex_match():
    context = mapnik.Context()
    context.push('name')
    f = mapnik.Feature(context, 0)
    f["name"] = 'Québec'
    expr = mapnik.Expression("[name].match('Québec')")
    eq_(expr.evaluate(f), True)  # 1 == True


def test_regex_replace():
    context = mapnik.Context()
    context.push('name')
    f = mapnik.Feature(context, 0)
    f["name"] = 'test'
    expr = mapnik.Expression("[name].replace('(\B)|( )','$1 ')")
    eq_(expr.evaluate(f), 't e s t')


def test_unicode_regex_replace_to_str():
    expr = mapnik.Expression("[name].replace('(\B)|( )','$1 ')")
    eq_(str(expr), "[name].replace('(\B)|( )','$1 ')")


def test_unicode_regex_replace():
    context = mapnik.Context()
    context.push('name')
    f = mapnik.Feature(context, 0)
    f["name"] = 'Québec'
    expr = mapnik.Expression("[name].replace('(\B)|( )','$1 ')")
    # will fail if -DBOOST_REGEX_HAS_ICU is not defined
    eq_(expr.evaluate(f), u'Q u é b e c')


def test_float_precision():
    context = mapnik.Context()
    context.push('num')
    f = mapnik.Feature(context, 0)
    f["num1"] = 1.0000
    f["num2"] = 1.0001
    eq_(f["num1"], 1.0000)
    eq_(f["num2"], 1.0001)
    expr = mapnik.Expression("[num1] = 1.0000")
    eq_(expr.evaluate(f), True)
    expr = mapnik.Expression("[num1].match('1')")
    eq_(expr.evaluate(f), True)
    expr = mapnik.Expression("[num2] = 1.0001")
    eq_(expr.evaluate(f), True)
    expr = mapnik.Expression("[num2].match('1.0001')")
    eq_(expr.evaluate(f), True)


def test_string_matching_on_precision():
    context = mapnik.Context()
    context.push('num')
    f = mapnik.Feature(context, 0)
    f["num"] = "1.0000"
    eq_(f["num"], "1.0000")
    expr = mapnik.Expression("[num].match('.*(^0|00)$')")
    eq_(expr.evaluate(f), True)


def test_creation_of_null_value():
    context = mapnik.Context()
    context.push('nv')
    f = mapnik.Feature(context, 0)
    f["nv"] = None
    eq_(f["nv"], None)
    eq_(f["nv"] is None, True)
    # test boolean
    f["nv"] = 0
    eq_(f["nv"], 0)
    eq_(f["nv"] is not None, True)


def test_creation_of_bool():
    context = mapnik.Context()
    context.push('bool')
    f = mapnik.Feature(context, 0)
    f["bool"] = True
    eq_(f["bool"], True)
    # TODO - will become int of 1 do to built in boost python conversion
    # https://github.com/mapnik/mapnik/issues/1873
    eq_(isinstance(f["bool"], bool) or isinstance(f["bool"], long), True)
    f["bool"] = False
    eq_(f["bool"], False)
    eq_(isinstance(f["bool"], bool) or isinstance(f["bool"], long), True)
    # test NoneType
    f["bool"] = None
    eq_(f["bool"], None)
    eq_(isinstance(f["bool"], bool) or isinstance(f["bool"], long), False)
    # test integer
    f["bool"] = 0
    eq_(f["bool"], 0)
    # https://github.com/mapnik/mapnik/issues/1873
    # ugh, boost_python's built into converter does not work right
    # eq_(isinstance(f["bool"],bool),False)

null_equality = [
    ['hello', False, unicode],
    [u'', False, unicode],
    [0, False, long],
    [123, False, long],
    [0.0, False, float],
    [123.123, False, float],
    [.1, False, float],
    # TODO - should become bool: https://github.com/mapnik/mapnik/issues/1873
    [False, False, long],
    # TODO - should become bool: https://github.com/mapnik/mapnik/issues/1873
    [True, False, long],
    [None, True, None],
    [2147483648, False, long],
    [922337203685477580, False, long]
]


def test_expressions_with_null_equality():
    for eq in null_equality:
        context = mapnik.Context()
        f = mapnik.Feature(context, 0)
        f["prop"] = eq[0]
        eq_(f["prop"], eq[0])
        if eq[0] is None:
            eq_(f["prop"] is None, True)
        else:
            eq_(isinstance(f['prop'], eq[2]), True,
                '%s is not an instance of %s' % (f['prop'], eq[2]))
        expr = mapnik.Expression("[prop] = null")
        eq_(expr.evaluate(f), eq[1])
        expr = mapnik.Expression("[prop] is null")
        eq_(expr.evaluate(f), eq[1])


def test_expressions_with_null_equality2():
    for eq in null_equality:
        context = mapnik.Context()
        f = mapnik.Feature(context, 0)
        f["prop"] = eq[0]
        eq_(f["prop"], eq[0])
        if eq[0] is None:
            eq_(f["prop"] is None, True)
        else:
            eq_(isinstance(f['prop'], eq[2]), True,
                '%s is not an instance of %s' % (f['prop'], eq[2]))
        # TODO - support `is not` syntax:
        # https://github.com/mapnik/mapnik/issues/796
        expr = mapnik.Expression("not [prop] is null")
        eq_(expr.evaluate(f), not eq[1])
        # https://github.com/mapnik/mapnik/issues/1642
        expr = mapnik.Expression("[prop] != null")
        eq_(expr.evaluate(f), not eq[1])

truthyness = [
    [u'hello', True, unicode],
    [u'', False, unicode],
    [0, False, long],
    [123, True, long],
    [0.0, False, float],
    [123.123, True, float],
    [.1, True, float],
    # TODO - should become bool: https://github.com/mapnik/mapnik/issues/1873
    [False, False, long],
    # TODO - should become bool: https://github.com/mapnik/mapnik/issues/1873
    [True, True, long],
    [None, False, None],
    [2147483648, True, long],
    [922337203685477580, True, long]
]


def test_expressions_for_thruthyness():
    context = mapnik.Context()
    for eq in truthyness:
        f = mapnik.Feature(context, 0)
        f["prop"] = eq[0]
        eq_(f["prop"], eq[0])
        if eq[0] is None:
            eq_(f["prop"] is None, True)
        else:
            eq_(isinstance(f['prop'], eq[2]), True,
                '%s is not an instance of %s' % (f['prop'], eq[2]))
        expr = mapnik.Expression("[prop]")
        eq_(expr.to_bool(f), eq[1])
        expr = mapnik.Expression("not [prop]")
        eq_(expr.to_bool(f), not eq[1])
        expr = mapnik.Expression("! [prop]")
        eq_(expr.to_bool(f), not eq[1])
    # also test if feature does not have property at all
    f2 = mapnik.Feature(context, 1)
    # no property existing will return value_null since
    # https://github.com/mapnik/mapnik/commit/562fada9d0f680f59b2d9f396c95320a0d753479#include/mapnik/feature.hpp
    eq_(f2["prop"] is None, True)
    expr = mapnik.Expression("[prop]")
    eq_(expr.evaluate(f2), None)
    eq_(expr.to_bool(f2), False)

# https://github.com/mapnik/mapnik/issues/1859


def test_if_null_and_empty_string_are_equal():
    context = mapnik.Context()
    f = mapnik.Feature(context, 0)
    f["empty"] = u""
    f["null"] = None
    # ensure base assumptions are good
    eq_(mapnik.Expression("[empty] = ''").to_bool(f), True)
    eq_(mapnik.Expression("[null] = null").to_bool(f), True)
    eq_(mapnik.Expression("[empty] != ''").to_bool(f), False)
    eq_(mapnik.Expression("[null] != null").to_bool(f), False)
    # now test expected behavior
    eq_(mapnik.Expression("[null] = ''").to_bool(f), False)
    eq_(mapnik.Expression("[empty] = null").to_bool(f), False)
    eq_(mapnik.Expression("[empty] != null").to_bool(f), True)
    # this one is the back compatibility shim
    eq_(mapnik.Expression("[null] != ''").to_bool(f), False)


def test_filtering_nulls_and_empty_strings():
    context = mapnik.Context()
    f = mapnik.Feature(context, 0)
    f["prop"] = u"hello"
    eq_(f["prop"], u"hello")
    eq_(mapnik.Expression("[prop]").to_bool(f), True)
    eq_(mapnik.Expression("! [prop]").to_bool(f), False)
    eq_(mapnik.Expression("[prop] != null").to_bool(f), True)
    eq_(mapnik.Expression("[prop] != ''").to_bool(f), True)
    eq_(mapnik.Expression("[prop] != null and [prop] != ''").to_bool(f), True)
    eq_(mapnik.Expression("[prop] != null or [prop] != ''").to_bool(f), True)
    f["prop2"] = u""
    eq_(f["prop2"], u"")
    eq_(mapnik.Expression("[prop2]").to_bool(f), False)
    eq_(mapnik.Expression("! [prop2]").to_bool(f), True)
    eq_(mapnik.Expression("[prop2] != null").to_bool(f), True)
    eq_(mapnik.Expression("[prop2] != ''").to_bool(f), False)
    eq_(mapnik.Expression("[prop2] = ''").to_bool(f), True)
    eq_(mapnik.Expression("[prop2] != null or [prop2] != ''").to_bool(f), True)
    eq_(mapnik.Expression(
        "[prop2] != null and [prop2] != ''").to_bool(f), False)
    f["prop3"] = None
    eq_(f["prop3"], None)
    eq_(mapnik.Expression("[prop3]").to_bool(f), False)
    eq_(mapnik.Expression("! [prop3]").to_bool(f), True)
    eq_(mapnik.Expression("[prop3] != null").to_bool(f), False)
    eq_(mapnik.Expression("[prop3] = null").to_bool(f), True)

    # https://github.com/mapnik/mapnik/issues/1859
    #eq_(mapnik.Expression("[prop3] != ''").to_bool(f),True)
    eq_(mapnik.Expression("[prop3] != ''").to_bool(f), False)

    eq_(mapnik.Expression("[prop3] = ''").to_bool(f), False)

    # https://github.com/mapnik/mapnik/issues/1859
    #eq_(mapnik.Expression("[prop3] != null or [prop3] != ''").to_bool(f),True)
    eq_(mapnik.Expression(
        "[prop3] != null or [prop3] != ''").to_bool(f), False)

    eq_(mapnik.Expression(
        "[prop3] != null and [prop3] != ''").to_bool(f), False)
    # attr not existing should behave the same as prop3
    eq_(mapnik.Expression("[prop4]").to_bool(f), False)
    eq_(mapnik.Expression("! [prop4]").to_bool(f), True)
    eq_(mapnik.Expression("[prop4] != null").to_bool(f), False)
    eq_(mapnik.Expression("[prop4] = null").to_bool(f), True)

    # https://github.com/mapnik/mapnik/issues/1859
    ##eq_(mapnik.Expression("[prop4] != ''").to_bool(f),True)
    eq_(mapnik.Expression("[prop4] != ''").to_bool(f), False)

    eq_(mapnik.Expression("[prop4] = ''").to_bool(f), False)

    # https://github.com/mapnik/mapnik/issues/1859
    ##eq_(mapnik.Expression("[prop4] != null or [prop4] != ''").to_bool(f),True)
    eq_(mapnik.Expression(
        "[prop4] != null or [prop4] != ''").to_bool(f), False)

    eq_(mapnik.Expression(
        "[prop4] != null and [prop4] != ''").to_bool(f), False)
    f["prop5"] = False
    eq_(f["prop5"], False)
    eq_(mapnik.Expression("[prop5]").to_bool(f), False)
    eq_(mapnik.Expression("! [prop5]").to_bool(f), True)
    eq_(mapnik.Expression("[prop5] != null").to_bool(f), True)
    eq_(mapnik.Expression("[prop5] = null").to_bool(f), False)
    eq_(mapnik.Expression("[prop5] != ''").to_bool(f), True)
    eq_(mapnik.Expression("[prop5] = ''").to_bool(f), False)
    eq_(mapnik.Expression("[prop5] != null or [prop5] != ''").to_bool(f), True)
    eq_(mapnik.Expression(
        "[prop5] != null and [prop5] != ''").to_bool(f), True)
    # note, we need to do [prop5] != 0 here instead of false due to this bug:
    # https://github.com/mapnik/mapnik/issues/1873
    eq_(mapnik.Expression(
        "[prop5] != null and [prop5] != '' and [prop5] != 0").to_bool(f), False)

# https://github.com/mapnik/mapnik/issues/1872


def test_falseyness_comparision():
    context = mapnik.Context()
    f = mapnik.Feature(context, 0)
    f["prop"] = 0
    eq_(mapnik.Expression("[prop]").to_bool(f), False)
    eq_(mapnik.Expression("[prop] = false").to_bool(f), True)
    eq_(mapnik.Expression("not [prop] != false").to_bool(f), True)
    eq_(mapnik.Expression("not [prop] = true").to_bool(f), True)
    eq_(mapnik.Expression("[prop] = true").to_bool(f), False)
    eq_(mapnik.Expression("[prop] != true").to_bool(f), True)

# https://github.com/mapnik/mapnik/issues/1806, fixed by
# https://github.com/mapnik/mapnik/issues/1872


def test_truthyness_comparision():
    context = mapnik.Context()
    f = mapnik.Feature(context, 0)
    f["prop"] = 1
    eq_(mapnik.Expression("[prop]").to_bool(f), True)
    eq_(mapnik.Expression("[prop] = false").to_bool(f), False)
    eq_(mapnik.Expression("not [prop] != false").to_bool(f), False)
    eq_(mapnik.Expression("not [prop] = true").to_bool(f), False)
    eq_(mapnik.Expression("[prop] = true").to_bool(f), True)
    eq_(mapnik.Expression("[prop] != true").to_bool(f), False)


def test_division_by_zero():
    expr = mapnik.Expression('[a]/[b]')
    c = mapnik.Context()
    c.push('a')
    c.push('b')
    f = mapnik.Feature(c, 0)
    f['a'] = 1
    f['b'] = 0
    eq_(expr.evaluate(f), None)


@raises(RuntimeError)
def test_invalid_syntax1():
    mapnik.Expression('abs()')


if __name__ == "__main__":
    exit(run_all(eval(x) for x in dir() if x.startswith("test_")))



#!/usr/bin/env python
from nose.tools import eq_

import mapnik

from .utilities import run_all


def test_logger_init():
    eq_(mapnik.severity_type.Debug, 0)
    eq_(mapnik.severity_type.Warn, 1)
    eq_(mapnik.severity_type.Error, 2)
    eq_(getattr(mapnik.severity_type, "None"), 3)
    default = mapnik.logger.get_severity()
    mapnik.logger.set_severity(mapnik.severity_type.Debug)
    eq_(mapnik.logger.get_severity(), mapnik.severity_type.Debug)
    mapnik.logger.set_severity(default)
    eq_(mapnik.logger.get_severity(), default)

if __name__ == "__main__":
    exit(run_all(eval(x) for x in dir() if x.startswith("test_")))



from nose.tools import eq_

import mapnik

from .utilities import run_all


def test_grayscale_conversion():
    im = mapnik.Image(2, 2)
    im.fill(mapnik.Color('white'))
    im.set_grayscale_to_alpha()
    pixel = im.get_pixel(0, 0)
    eq_((pixel >> 24) & 0xff, 255)

if __name__ == "__main__":
    exit(run_all(eval(x) for x in dir() if x.startswith("test_")))



#!/usr/bin/env python

from __future__ import print_function

import os
import shutil

from nose.tools import eq_

import mapnik

from .utilities import execution_path, run_all


def setup():
    # All of the paths used are relative, if we run the tests
    # from another directory we need to chdir()
    os.chdir(execution_path('.'))


def make_tmp_map():
    m = mapnik.Map(512, 512)
    m.background_color = mapnik.Color('steelblue')
    ds = mapnik.MemoryDatasource()
    context = mapnik.Context()
    context.push('Name')
    f = mapnik.Feature(context, 1)
    f['Name'] = 'Hello'
    f.geometry = mapnik.Geometry.from_wkt('POINT (0 0)')
    ds.add_feature(f)
    s = mapnik.Style()
    r = mapnik.Rule()
    sym = mapnik.MarkersSymbolizer()
    sym.allow_overlap = True
    r.symbols.append(sym)
    s.rules.append(r)
    lyr = mapnik.Layer('Layer')
    lyr.datasource = ds
    lyr.styles.append('style')
    m.append_style('style', s)
    m.layers.append(lyr)
    return m


def draw_title(m, ctx, text, size=10, color=mapnik.Color('black')):
    """ Draw a Map Title near the top of a page."""
    middle = m.width / 2.0
    ctx.set_source_rgba(*cairo_color(color))
    ctx.select_font_face(
        "DejaVu Sans Book",
        cairo.FONT_SLANT_NORMAL,
        cairo.FONT_WEIGHT_NORMAL)
    ctx.set_font_size(size)
    x_bearing, y_bearing, width, height = ctx.text_extents(text)[:4]
    ctx.move_to(middle - width / 2 - x_bearing, 20.0 - height / 2 - y_bearing)
    ctx.show_text(text)


def draw_neatline(m, ctx):
    w, h = m.width, m.height
    ctx.set_source_rgba(*cairo_color(mapnik.Color('black')))
    outline = [
        [0, 0], [w, 0], [w, h], [0, h]
    ]
    ctx.set_line_width(1)
    for idx, pt in enumerate(outline):
        if (idx == 0):
            ctx.move_to(*pt)
        else:
            ctx.line_to(*pt)
    ctx.close_path()
    inset = 6
    inline = [
        [inset, inset], [w - inset, inset], [w -
                                             inset, h - inset], [inset, h - inset]
    ]
    ctx.set_line_width(inset / 2)
    for idx, pt in enumerate(inline):
        if (idx == 0):
            ctx.move_to(*pt)
        else:
            ctx.line_to(*pt)
    ctx.close_path()
    ctx.stroke()


def cairo_color(c):
    """ Return a Cairo color tuple from a Mapnik Color."""
    ctx_c = (c.r / 255.0, c.g / 255.0, c.b / 255.0, c.a / 255.0)
    return ctx_c

if mapnik.has_pycairo():
    import cairo

    def test_passing_pycairo_context_svg():
        m = make_tmp_map()
        m.zoom_to_box(mapnik.Box2d(-180, -90, 180, 90))
        test_cairo_file = '/tmp/mapnik-cairo-context-test.svg'
        surface = cairo.SVGSurface(test_cairo_file, m.width, m.height)
        expected_cairo_file = './images/pycairo/cairo-cairo-expected.svg'
        context = cairo.Context(surface)
        mapnik.render(m, context)
        draw_title(m, context, "Hello Map", size=20)
        draw_neatline(m, context)
        surface.finish()
        if not os.path.exists(expected_cairo_file) or os.environ.get('UPDATE'):
            print('generated expected cairo surface file', expected_cairo_file)
            shutil.copy(test_cairo_file, expected_cairo_file)
        diff = abs(
            os.stat(expected_cairo_file).st_size -
            os.stat(test_cairo_file).st_size)
        msg = 'diff in size (%s) between actual (%s) and expected(%s)' % (
            diff, test_cairo_file, 'tests/python_tests/' + expected_cairo_file)
        eq_(diff < 1500, True, msg)
        os.remove(test_cairo_file)

    def test_passing_pycairo_context_pdf():
        m = make_tmp_map()
        m.zoom_to_box(mapnik.Box2d(-180, -90, 180, 90))
        test_cairo_file = '/tmp/mapnik-cairo-context-test.pdf'
        surface = cairo.PDFSurface(test_cairo_file, m.width, m.height)
        expected_cairo_file = './images/pycairo/cairo-cairo-expected.pdf'
        context = cairo.Context(surface)
        mapnik.render(m, context)
        draw_title(m, context, "Hello Map", size=20)
        draw_neatline(m, context)
        surface.finish()
        if not os.path.exists(expected_cairo_file) or os.environ.get('UPDATE'):
            print('generated expected cairo surface file', expected_cairo_file)
            shutil.copy(test_cairo_file, expected_cairo_file)
        diff = abs(
            os.stat(expected_cairo_file).st_size -
            os.stat(test_cairo_file).st_size)
        msg = 'diff in size (%s) between actual (%s) and expected(%s)' % (
            diff, test_cairo_file, 'tests/python_tests/' + expected_cairo_file)
        eq_(diff < 1500, True, msg)
        os.remove(test_cairo_file)

    def test_passing_pycairo_context_png():
        m = make_tmp_map()
        m.zoom_to_box(mapnik.Box2d(-180, -90, 180, 90))
        test_cairo_file = '/tmp/mapnik-cairo-context-test.png'
        surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, m.width, m.height)
        expected_cairo_file = './images/pycairo/cairo-cairo-expected.png'
        expected_cairo_file2 = './images/pycairo/cairo-cairo-expected-reduced.png'
        context = cairo.Context(surface)
        mapnik.render(m, context)
        draw_title(m, context, "Hello Map", size=20)
        draw_neatline(m, context)
        surface.write_to_png(test_cairo_file)
        reduced_color_image = test_cairo_file.replace('png', '-mapnik.png')
        im = mapnik.Image.from_cairo(surface)
        im.save(reduced_color_image, 'png8')
        surface.finish()
        if not os.path.exists(expected_cairo_file) or os.environ.get('UPDATE'):
            print('generated expected cairo surface file', expected_cairo_file)
            shutil.copy(test_cairo_file, expected_cairo_file)
        diff = abs(
            os.stat(expected_cairo_file).st_size -
            os.stat(test_cairo_file).st_size)
        msg = 'diff in size (%s) between actual (%s) and expected(%s)' % (
            diff, test_cairo_file, 'tests/python_tests/' + expected_cairo_file)
        eq_(diff < 500, True, msg)
        os.remove(test_cairo_file)
        if not os.path.exists(
                expected_cairo_file2) or os.environ.get('UPDATE'):
            print(
                'generated expected cairo surface file',
                expected_cairo_file2)
            shutil.copy(reduced_color_image, expected_cairo_file2)
        diff = abs(
            os.stat(expected_cairo_file2).st_size -
            os.stat(reduced_color_image).st_size)
        msg = 'diff in size (%s) between actual (%s) and expected(%s)' % (
            diff, reduced_color_image, 'tests/python_tests/' + expected_cairo_file2)
        eq_(diff < 500, True, msg)
        os.remove(reduced_color_image)

    if 'sqlite' in mapnik.DatasourceCache.plugin_names():
        def _pycairo_surface(type, sym):
            test_cairo_file = '/tmp/mapnik-cairo-surface-test.%s.%s' % (
                sym, type)
            expected_cairo_file = './images/pycairo/cairo-surface-expected.%s.%s' % (
                sym, type)
            m = mapnik.Map(256, 256)
            mapnik.load_map(m, '../data/good_maps/%s_symbolizer.xml' % sym)
            m.zoom_all()
            if hasattr(cairo, '%sSurface' % type.upper()):
                surface = getattr(
                    cairo,
                    '%sSurface' %
                    type.upper())(
                    test_cairo_file,
                    m.width,
                    m.height)
                mapnik.render(m, surface)
                surface.finish()
                if not os.path.exists(
                        expected_cairo_file) or os.environ.get('UPDATE'):
                    print(
                        'generated expected cairo surface file',
                        expected_cairo_file)
                    shutil.copy(test_cairo_file, expected_cairo_file)
                diff = abs(
                    os.stat(expected_cairo_file).st_size -
                    os.stat(test_cairo_file).st_size)
                msg = 'diff in size (%s) between actual (%s) and expected(%s)' % (
                    diff, test_cairo_file, 'tests/python_tests/' + expected_cairo_file)
                if os.uname()[0] == 'Darwin':
                    eq_(diff < 2100, True, msg)
                else:
                    eq_(diff < 23000, True, msg)
                os.remove(test_cairo_file)
                return True
            else:
                print(
                    'skipping cairo.%s test since surface is not available' %
                    type.upper())
                return True

        def test_pycairo_svg_surface1():
            eq_(_pycairo_surface('svg', 'point'), True)

        def test_pycairo_svg_surface2():
            eq_(_pycairo_surface('svg', 'building'), True)

        def test_pycairo_svg_surface3():
            eq_(_pycairo_surface('svg', 'polygon'), True)

        def test_pycairo_pdf_surface1():
            eq_(_pycairo_surface('pdf', 'point'), True)

        def test_pycairo_pdf_surface2():
            eq_(_pycairo_surface('pdf', 'building'), True)

        def test_pycairo_pdf_surface3():
            eq_(_pycairo_surface('pdf', 'polygon'), True)

if __name__ == "__main__":
    setup()
    exit(run_all(eval(x) for x in dir() if x.startswith("test_")))



#!/usr/bin/env python

import os

from nose.tools import eq_

import mapnik

from .utilities import execution_path, run_all

try:
    import itertools.izip as zip
except ImportError:
    pass


def setup():
    # All of the paths used are relative, if we run the tests
    # from another directory we need to chdir()
    os.chdir(execution_path('.'))


def compare_shape_between_mapnik_and_ogr(shapefile, query=None):
    plugins = mapnik.DatasourceCache.plugin_names()
    if 'shape' in plugins and 'ogr' in plugins:
        ds1 = mapnik.Ogr(file=shapefile, layer_by_index=0)
        ds2 = mapnik.Shapefile(file=shapefile)
        if query:
            fs1 = ds1.features(query)
            fs2 = ds2.features(query)
        else:
            fs1 = ds1.featureset()
            fs2 = ds2.featureset()
        count = 0
        for feat1, feat2 in zip(fs1, fs2):
            count += 1
            eq_(feat1.id(), feat2.id(),
                '%s : ogr feature id %s "%s" does not equal shapefile feature id %s "%s"'
                % (count, feat1.id(), str(feat1.attributes), feat2.id(), str(feat2.attributes)))
    return True


def test_shapefile_line_featureset_id():
    compare_shape_between_mapnik_and_ogr('../data/shp/polylines.shp')


def test_shapefile_polygon_featureset_id():
    compare_shape_between_mapnik_and_ogr('../data/shp/poly.shp')


def test_shapefile_polygon_feature_query_id():
    bbox = (15523428.2632, 4110477.6323, -11218494.8310, 7495720.7404)
    query = mapnik.Query(mapnik.Box2d(*bbox))
    if 'ogr' in mapnik.DatasourceCache.plugin_names():
        ds = mapnik.Ogr(file='../data/shp/world_merc.shp', layer_by_index=0)
        for fld in ds.fields():
            query.add_property_name(fld)
        compare_shape_between_mapnik_and_ogr(
            '../data/shp/world_merc.shp', query)


def test_feature_hit_count():
    pass
    #raise Todo("need to optimize multigeom bbox handling in shapeindex: https://github.com/mapnik/mapnik/issues/783")
    # results in different results between shp and ogr!
    #bbox = (-14284551.8434, 2074195.1992, -7474929.8687, 8140237.7628)
    #bbox = (1113194.91,4512803.085,2226389.82,6739192.905)
    #query = mapnik.Query(mapnik.Box2d(*bbox))
    # if 'ogr' in mapnik.DatasourceCache.plugin_names():
    #    ds1 = mapnik.Ogr(file='../data/shp/world_merc.shp',layer_by_index=0)
    #    for fld in ds1.fields():
    #        query.add_property_name(fld)
    #    ds2 = mapnik.Shapefile(file='../data/shp/world_merc.shp')
    #    count1 = len(ds1.features(query).features)
    #    count2 = len(ds2.features(query).features)
    #    eq_(count1,count2,"Feature count differs between OGR driver (%s features) and Shapefile Driver (%s features) when querying the same bbox" % (count1,count2))

if __name__ == "__main__":
    setup()
    exit(run_all(eval(x) for x in dir() if x.startswith("test_")))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os

from nose.tools import assert_almost_equal, eq_, raises

import mapnik

from .utilities import execution_path, run_all


def setup():
    # All of the paths used are relative, if we run the tests
    # from another directory we need to chdir()
    os.chdir(execution_path('.'))

if 'shape' in mapnik.DatasourceCache.plugin_names():

    # Shapefile initialization
    def test_shapefile_init():
        s = mapnik.Shapefile(file='../data/shp/boundaries')

        e = s.envelope()

        assert_almost_equal(e.minx, -11121.6896651, places=7)
        assert_almost_equal(e.miny, -724724.216526, places=6)
        assert_almost_equal(e.maxx, 2463000.67866, places=5)
        assert_almost_equal(e.maxy, 1649661.267, places=3)

    # Shapefile properties
    def test_shapefile_properties():
        s = mapnik.Shapefile(file='../data/shp/boundaries', encoding='latin1')
        f = list(s.features_at_point(s.envelope().center()))[0]

        eq_(f['CGNS_FID'], u'6f733341ba2011d892e2080020a0f4c9')
        eq_(f['COUNTRY'], u'CAN')
        eq_(f['F_CODE'], u'FA001')
        eq_(f['NAME_EN'], u'Quebec')
        # this seems to break if icu data linking is not working
        eq_(f['NOM_FR'], u'Qu\xe9bec')
        eq_(f['NOM_FR'], u'Québec')
        eq_(f['Shape_Area'], 1512185733150.0)
        eq_(f['Shape_Leng'], 19218883.724300001)

    @raises(RuntimeError)
    def test_that_nonexistant_query_field_throws(**kwargs):
        ds = mapnik.Shapefile(file='../data/shp/world_merc')
        eq_(len(ds.fields()), 11)
        eq_(ds.fields(), ['FIPS', 'ISO2', 'ISO3', 'UN', 'NAME',
                          'AREA', 'POP2005', 'REGION', 'SUBREGION', 'LON', 'LAT'])
        eq_(ds.field_types(),
            ['str',
             'str',
             'str',
             'int',
             'str',
             'int',
             'int',
             'int',
             'int',
             'float',
             'float'])
        query = mapnik.Query(ds.envelope())
        for fld in ds.fields():
            query.add_property_name(fld)
        # also add an invalid one, triggering throw
        query.add_property_name('bogus')
        ds.features(query)

    def test_dbf_logical_field_is_boolean():
        ds = mapnik.Shapefile(file='../data/shp/long_lat')
        eq_(len(ds.fields()), 7)
        eq_(ds.fields(), ['LONG', 'LAT', 'LOGICAL_TR',
                          'LOGICAL_FA', 'CHARACTER', 'NUMERIC', 'DATE'])
        eq_(ds.field_types(), ['str', 'str',
                               'bool', 'bool', 'str', 'float', 'str'])
        query = mapnik.Query(ds.envelope())
        for fld in ds.fields():
            query.add_property_name(fld)
        feat = list(ds.all_features())[0]
        eq_(feat.id(), 1)
        eq_(feat['LONG'], '0')
        eq_(feat['LAT'], '0')
        eq_(feat['LOGICAL_TR'], True)
        eq_(feat['LOGICAL_FA'], False)
        eq_(feat['CHARACTER'], '254')
        eq_(feat['NUMERIC'], 32)
        eq_(feat['DATE'], '20121202')

    # created by hand in qgis 1.8.0
    def test_shapefile_point2d_from_qgis():
        ds = mapnik.Shapefile(file='../data/shp/points/qgis.shp')
        eq_(len(ds.fields()), 2)
        eq_(ds.fields(), ['id', 'name'])
        eq_(ds.field_types(), ['int', 'str'])
        eq_(len(list(ds.all_features())), 3)

    # ogr2ogr tests/data/shp/3dpoint/ogr_zfield.shp
    # tests/data/shp/3dpoint/qgis.shp -zfield id
    def test_shapefile_point_z_from_qgis():
        ds = mapnik.Shapefile(file='../data/shp/points/ogr_zfield.shp')
        eq_(len(ds.fields()), 2)
        eq_(ds.fields(), ['id', 'name'])
        eq_(ds.field_types(), ['int', 'str'])
        eq_(len(list(ds.all_features())), 3)

    def test_shapefile_multipoint_from_qgis():
        ds = mapnik.Shapefile(file='../data/shp/points/qgis_multi.shp')
        eq_(len(ds.fields()), 2)
        eq_(ds.fields(), ['id', 'name'])
        eq_(ds.field_types(), ['int', 'str'])
        eq_(len(list(ds.all_features())), 1)

    # pointzm from arcinfo
    def test_shapefile_point_zm_from_arcgis():
        ds = mapnik.Shapefile(file='../data/shp/points/poi.shp')
        eq_(len(ds.fields()), 7)
        eq_(ds.fields(),
            ['interst_id',
             'state_d',
             'cnty_name',
             'latitude',
             'longitude',
             'Name',
             'Website'])
        eq_(ds.field_types(), ['str', 'str',
                               'str', 'float', 'float', 'str', 'str'])
        eq_(len(list(ds.all_features())), 17)

    # copy of the above with ogr2ogr that makes m record 14 instead of 18
    def test_shapefile_point_zm_from_ogr():
        ds = mapnik.Shapefile(file='../data/shp/points/poi_ogr.shp')
        eq_(len(ds.fields()), 7)
        eq_(ds.fields(),
            ['interst_id',
             'state_d',
             'cnty_name',
             'latitude',
             'longitude',
             'Name',
             'Website'])
        eq_(ds.field_types(), ['str', 'str',
                               'str', 'float', 'float', 'str', 'str'])
        eq_(len(list(ds.all_features())), 17)

if __name__ == "__main__":
    setup()
    exit(run_all(eval(x) for x in dir() if x.startswith("test_")))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os

from nose.tools import eq_

import mapnik

from .utilities import execution_path, run_all


def setup():
    # All of the paths used are relative, if we run the tests
    # from another directory we need to chdir()
    os.chdir(execution_path('.'))

if 'osm' in mapnik.DatasourceCache.plugin_names():

    # osm initialization
    def test_osm_init():
        ds = mapnik.Osm(file='../data/osm/nodes.osm')

        e = ds.envelope()

        # these are hardcoded in the plugin… ugh
        eq_(e.minx >= -180.0, True)
        eq_(e.miny >= -90.0, True)
        eq_(e.maxx <= 180.0, True)
        eq_(e.maxy <= 90, True)

    def test_that_nonexistant_query_field_throws(**kwargs):
        ds = mapnik.Osm(file='../data/osm/nodes.osm')
        eq_(len(ds.fields()), 0)
        query = mapnik.Query(ds.envelope())
        for fld in ds.fields():
            query.add_property_name(fld)
        # also add an invalid one, triggering throw
        query.add_property_name('bogus')
        ds.features(query)

    def test_that_64bit_int_fields_work():
        ds = mapnik.Osm(file='../data/osm/64bit.osm')
        eq_(len(ds.fields()), 4)
        eq_(ds.fields(), ['bigint', 'highway', 'junction', 'note'])
        eq_(ds.field_types(), ['str', 'str', 'str', 'str'])
        fs = ds.featureset()
        feat = fs.next()
        eq_(feat.to_geojson(
        ), '{"type":"Feature","id":890,"geometry":{"type":"Point","coordinates":[-61.7960248,17.1415874]},"properties":{}}')
        eq_(feat.id(), 4294968186)
        eq_(feat['bigint'], None)
        feat = fs.next()
        eq_(feat['bigint'], '9223372036854775807')

    def test_reading_ways():
        ds = mapnik.Osm(file='../data/osm/ways.osm')
        eq_(len(ds.fields()), 0)
        eq_(ds.fields(), [])
        eq_(ds.field_types(), [])
        feat = ds.all_features()[4]
        eq_(feat.to_geojson(
        ), '{"type":"Feature","id":1,"geometry":{"type":"LineString","coordinates":[[0,2],[0,-2]]},"properties":{}}')
        eq_(feat.id(), 1)


if __name__ == "__main__":
    setup()
    exit(run_all(eval(x) for x in dir() if x.startswith("test_")))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys

from nose.tools import assert_almost_equal, eq_, raises

import mapnik

from .utilities import READ_FLAGS, execution_path, get_unique_colors, run_all

PYTHON3 = sys.version_info[0] == 3
if PYTHON3:
    buffer = memoryview


def setup():
    # All of the paths used are relative, if we run the tests
    # from another directory we need to chdir()
    os.chdir(execution_path('.'))


def test_type():
    im = mapnik.Image(256, 256)
    eq_(im.get_type(), mapnik.ImageType.rgba8)
    im = mapnik.Image(256, 256, mapnik.ImageType.gray8)
    eq_(im.get_type(), mapnik.ImageType.gray8)


def test_image_premultiply():
    im = mapnik.Image(256, 256)
    eq_(im.premultiplied(), False)
    # Premultiply should return true that it worked
    eq_(im.premultiply(), True)
    eq_(im.premultiplied(), True)
    # Premultipling again should return false as nothing should happen
    eq_(im.premultiply(), False)
    eq_(im.premultiplied(), True)
    # Demultiply should return true that it worked
    eq_(im.demultiply(), True)
    eq_(im.premultiplied(), False)
    # Demultiply again should not work and return false as it did nothing
    eq_(im.demultiply(), False)
    eq_(im.premultiplied(), False)


def test_image_premultiply_values():
    im = mapnik.Image(256, 256)
    im.fill(mapnik.Color(16, 33, 255, 128))
    im.premultiply()
    c = im.get_pixel(0, 0, True)
    eq_(c.r, 8)
    eq_(c.g, 17)
    eq_(c.b, 128)
    eq_(c.a, 128)
    im.demultiply()
    # Do to the nature of this operation the result will not be exactly the
    # same
    c = im.get_pixel(0, 0, True)
    eq_(c.r, 15)
    eq_(c.g, 33)
    eq_(c.b, 255)
    eq_(c.a, 128)


def test_apply_opacity():
    im = mapnik.Image(4, 4)
    im.fill(mapnik.Color(128, 128, 128, 128))
    im.apply_opacity(0.75)
    c = im.get_pixel(0, 0, True)
    eq_(c.r, 128)
    eq_(c.g, 128)
    eq_(c.b, 128)
    eq_(c.a, 96)


def test_background():
    im = mapnik.Image(256, 256)
    eq_(im.premultiplied(), False)
    im.fill(mapnik.Color(32, 64, 125, 128))
    eq_(im.premultiplied(), False)
    c = im.get_pixel(0, 0, True)
    eq_(c.get_premultiplied(), False)
    eq_(c.r, 32)
    eq_(c.g, 64)
    eq_(c.b, 125)
    eq_(c.a, 128)
    # Now again with a premultiplied alpha
    im.fill(mapnik.Color(32, 64, 125, 128, True))
    eq_(im.premultiplied(), True)
    c = im.get_pixel(0, 0, True)
    eq_(c.get_premultiplied(), True)
    eq_(c.r, 32)
    eq_(c.g, 64)
    eq_(c.b, 125)
    eq_(c.a, 128)


def test_set_and_get_pixel():
    # Create an image that is not premultiplied
    im = mapnik.Image(256, 256)
    c0 = mapnik.Color(16, 33, 255, 128)
    c0_pre = mapnik.Color(16, 33, 255, 128, True)
    im.set_pixel(0, 0, c0)
    im.set_pixel(1, 1, c0_pre)
    # No differences for non premultiplied pixels
    c1_int = mapnik.Color(im.get_pixel(0, 0))
    eq_(c0.r, c1_int.r)
    eq_(c0.g, c1_int.g)
    eq_(c0.b, c1_int.b)
    eq_(c0.a, c1_int.a)
    c1 = im.get_pixel(0, 0, True)
    eq_(c0.r, c1.r)
    eq_(c0.g, c1.g)
    eq_(c0.b, c1.b)
    eq_(c0.a, c1.a)
    # The premultiplied Color should be demultiplied before being applied.
    c0_pre.demultiply()
    c1_int = mapnik.Color(im.get_pixel(1, 1))
    eq_(c0_pre.r, c1_int.r)
    eq_(c0_pre.g, c1_int.g)
    eq_(c0_pre.b, c1_int.b)
    eq_(c0_pre.a, c1_int.a)
    c1 = im.get_pixel(1, 1, True)
    eq_(c0_pre.r, c1.r)
    eq_(c0_pre.g, c1.g)
    eq_(c0_pre.b, c1.b)
    eq_(c0_pre.a, c1.a)

    # Now create a new image that is premultiplied
    im = mapnik.Image(256, 256, mapnik.ImageType.rgba8, True, True)
    c0 = mapnik.Color(16, 33, 255, 128)
    c0_pre = mapnik.Color(16, 33, 255, 128, True)
    im.set_pixel(0, 0, c0)
    im.set_pixel(1, 1, c0_pre)
    # It should have put pixels that are the same as premultiplied so
    # premultiply c0
    c0.premultiply()
    c1_int = mapnik.Color(im.get_pixel(0, 0))
    eq_(c0.r, c1_int.r)
    eq_(c0.g, c1_int.g)
    eq_(c0.b, c1_int.b)
    eq_(c0.a, c1_int.a)
    c1 = im.get_pixel(0, 0, True)
    eq_(c0.r, c1.r)
    eq_(c0.g, c1.g)
    eq_(c0.b, c1.b)
    eq_(c0.a, c1.a)
    # The premultiplied Color should be the same though
    c1_int = mapnik.Color(im.get_pixel(1, 1))
    eq_(c0_pre.r, c1_int.r)
    eq_(c0_pre.g, c1_int.g)
    eq_(c0_pre.b, c1_int.b)
    eq_(c0_pre.a, c1_int.a)
    c1 = im.get_pixel(1, 1, True)
    eq_(c0_pre.r, c1.r)
    eq_(c0_pre.g, c1.g)
    eq_(c0_pre.b, c1.b)
    eq_(c0_pre.a, c1.a)


def test_pixel_gray8():
    im = mapnik.Image(4, 4, mapnik.ImageType.gray8)
    val_list = range(20)
    for v in val_list:
        im.set_pixel(0, 0, v)
        eq_(im.get_pixel(0, 0), v)
        im.set_pixel(0, 0, -v)
        eq_(im.get_pixel(0, 0), 0)


def test_pixel_gray8s():
    im = mapnik.Image(4, 4, mapnik.ImageType.gray8s)
    val_list = range(20)
    for v in val_list:
        im.set_pixel(0, 0, v)
        eq_(im.get_pixel(0, 0), v)
        im.set_pixel(0, 0, -v)
        eq_(im.get_pixel(0, 0), -v)


def test_pixel_gray16():
    im = mapnik.Image(4, 4, mapnik.ImageType.gray16)
    val_list = range(20)
    for v in val_list:
        im.set_pixel(0, 0, v)
        eq_(im.get_pixel(0, 0), v)
        im.set_pixel(0, 0, -v)
        eq_(im.get_pixel(0, 0), 0)


def test_pixel_gray16s():
    im = mapnik.Image(4, 4, mapnik.ImageType.gray16s)
    val_list = range(20)
    for v in val_list:
        im.set_pixel(0, 0, v)
        eq_(im.get_pixel(0, 0), v)
        im.set_pixel(0, 0, -v)
        eq_(im.get_pixel(0, 0), -v)


def test_pixel_gray32():
    im = mapnik.Image(4, 4, mapnik.ImageType.gray32)
    val_list = range(20)
    for v in val_list:
        im.set_pixel(0, 0, v)
        eq_(im.get_pixel(0, 0), v)
        im.set_pixel(0, 0, -v)
        eq_(im.get_pixel(0, 0), 0)


def test_pixel_gray32s():
    im = mapnik.Image(4, 4, mapnik.ImageType.gray32s)
    val_list = range(20)
    for v in val_list:
        im.set_pixel(0, 0, v)
        eq_(im.get_pixel(0, 0), v)
        im.set_pixel(0, 0, -v)
        eq_(im.get_pixel(0, 0), -v)


def test_pixel_gray64():
    im = mapnik.Image(4, 4, mapnik.ImageType.gray64)
    val_list = range(20)
    for v in val_list:
        im.set_pixel(0, 0, v)
        eq_(im.get_pixel(0, 0), v)
        im.set_pixel(0, 0, -v)
        eq_(im.get_pixel(0, 0), 0)


def test_pixel_gray64s():
    im = mapnik.Image(4, 4, mapnik.ImageType.gray64s)
    val_list = range(20)
    for v in val_list:
        im.set_pixel(0, 0, v)
        eq_(im.get_pixel(0, 0), v)
        im.set_pixel(0, 0, -v)
        eq_(im.get_pixel(0, 0), -v)


def test_pixel_floats():
    im = mapnik.Image(4, 4, mapnik.ImageType.gray32f)
    val_list = [0.9, 0.99, 0.999, 0.9999, 0.99999, 1, 1.0001, 1.001, 1.01, 1.1]
    for v in val_list:
        im.set_pixel(0, 0, v)
        assert_almost_equal(im.get_pixel(0, 0), v)
        im.set_pixel(0, 0, -v)
        assert_almost_equal(im.get_pixel(0, 0), -v)


def test_pixel_doubles():
    im = mapnik.Image(4, 4, mapnik.ImageType.gray64f)
    val_list = [0.9, 0.99, 0.999, 0.9999, 0.99999, 1, 1.0001, 1.001, 1.01, 1.1]
    for v in val_list:
        im.set_pixel(0, 0, v)
        assert_almost_equal(im.get_pixel(0, 0), v)
        im.set_pixel(0, 0, -v)
        assert_almost_equal(im.get_pixel(0, 0), -v)


def test_pixel_overflow():
    im = mapnik.Image(4, 4, mapnik.ImageType.gray8)
    im.set_pixel(0, 0, 256)
    eq_(im.get_pixel(0, 0), 255)


def test_pixel_underflow():
    im = mapnik.Image(4, 4, mapnik.ImageType.gray8)
    im.set_pixel(0, 0, -1)
    eq_(im.get_pixel(0, 0), 0)
    im = mapnik.Image(4, 4, mapnik.ImageType.gray16)
    im.set_pixel(0, 0, -1)
    eq_(im.get_pixel(0, 0), 0)


@raises(IndexError)
def test_set_pixel_out_of_range_1():
    im = mapnik.Image(4, 4)
    c = mapnik.Color('blue')
    im.set_pixel(5, 5, c)


@raises(OverflowError)
def test_set_pixel_out_of_range_2():
    im = mapnik.Image(4, 4)
    c = mapnik.Color('blue')
    im.set_pixel(-1, 1, c)


@raises(IndexError)
def test_get_pixel_out_of_range_1():
    im = mapnik.Image(4, 4)
    c = im.get_pixel(5, 5)


@raises(OverflowError)
def test_get_pixel_out_of_range_2():
    im = mapnik.Image(4, 4)
    c = im.get_pixel(-1, 1)


@raises(IndexError)
def test_get_pixel_color_out_of_range_1():
    im = mapnik.Image(4, 4)
    c = im.get_pixel(5, 5, True)


@raises(OverflowError)
def test_get_pixel_color_out_of_range_2():
    im = mapnik.Image(4, 4)
    c = im.get_pixel(-1, 1, True)


def test_set_color_to_alpha():
    im = mapnik.Image(256, 256)
    im.fill(mapnik.Color('rgba(12,12,12,255)'))
    eq_(get_unique_colors(im), ['rgba(12,12,12,255)'])
    im.set_color_to_alpha(mapnik.Color('rgba(12,12,12,0)'))
    eq_(get_unique_colors(im), ['rgba(0,0,0,0)'])


@raises(RuntimeError)
def test_negative_image_dimensions():
    # TODO - this may have regressed in
    # https://github.com/mapnik/mapnik/commit/4f3521ac24b61fc8ae8fd344a16dc3a5fdf15af7
    im = mapnik.Image(-40, 40)
    # should not get here
    eq_(im.width(), 0)
    eq_(im.height(), 0)


def test_jpeg_round_trip():
    filepath = '/tmp/mapnik-jpeg-io.jpeg'
    im = mapnik.Image(255, 267)
    im.fill(mapnik.Color('rgba(1,2,3,.5)'))
    im.save(filepath, 'jpeg')
    im2 = mapnik.Image.open(filepath)
    with open(filepath, READ_FLAGS) as f:
        im3 = mapnik.Image.fromstring(f.read())
    eq_(im.width(), im2.width())
    eq_(im.height(), im2.height())
    eq_(im.width(), im3.width())
    eq_(im.height(), im3.height())
    eq_(len(im.tostring()), len(im2.tostring()))
    eq_(len(im.tostring('jpeg')), len(im2.tostring('jpeg')))
    eq_(len(im.tostring()), len(im3.tostring()))
    eq_(len(im.tostring('jpeg')), len(im3.tostring('jpeg')))


def test_png_round_trip():
    filepath = '/tmp/mapnik-png-io.png'
    im = mapnik.Image(255, 267)
    im.fill(mapnik.Color('rgba(1,2,3,.5)'))
    im.save(filepath, 'png')
    im2 = mapnik.Image.open(filepath)
    with open(filepath, READ_FLAGS) as f:
        im3 = mapnik.Image.fromstring(f.read())
    eq_(im.width(), im2.width())
    eq_(im.height(), im2.height())
    eq_(im.width(), im3.width())
    eq_(im.height(), im3.height())
    eq_(len(im.tostring()), len(im2.tostring()))
    eq_(len(im.tostring('png')), len(im2.tostring('png')))
    eq_(len(im.tostring('png8')), len(im2.tostring('png8')))
    eq_(len(im.tostring()), len(im3.tostring()))
    eq_(len(im.tostring('png')), len(im3.tostring('png')))
    eq_(len(im.tostring('png8')), len(im3.tostring('png8')))


def test_image_open_from_string():
    filepath = '../data/images/dummy.png'
    im1 = mapnik.Image.open(filepath)
    with open(filepath, READ_FLAGS) as f:
        im2 = mapnik.Image.fromstring(f.read())
    eq_(im1.width(), im2.width())
    length = len(im1.tostring())
    eq_(length, len(im2.tostring()))
    eq_(len(mapnik.Image.fromstring(im1.tostring('png')).tostring()), length)
    eq_(len(mapnik.Image.fromstring(im1.tostring('jpeg')).tostring()), length)
    eq_(len(mapnik.Image.frombuffer(buffer(im1.tostring('png'))).tostring()), length)
    eq_(len(mapnik.Image.frombuffer(buffer(im1.tostring('jpeg'))).tostring()), length)

    # TODO - https://github.com/mapnik/mapnik/issues/1831
    eq_(len(mapnik.Image.fromstring(im1.tostring('tiff')).tostring()), length)
    eq_(len(mapnik.Image.frombuffer(buffer(im1.tostring('tiff'))).tostring()), length)

if __name__ == "__main__":
    setup()
    exit(run_all(eval(x) for x in dir() if x.startswith("test_")))



# coding=utf8
import os

from nose.tools import eq_

import mapnik

from .utilities import execution_path, run_all


def setup():
    # All of the paths used are relative, if we run the tests
    # from another directory we need to chdir()
    os.chdir(execution_path('.'))

if 'shape' in mapnik.DatasourceCache.plugin_names():

    #@raises(RuntimeError)
    def test_zoom_all_will_fail():
        m = mapnik.Map(512, 512)
        mapnik.load_map(m, '../data/good_maps/wgs842merc_reprojection.xml')
        m.zoom_all()

    def test_zoom_all_will_work_with_max_extent():
        m = mapnik.Map(512, 512)
        mapnik.load_map(m, '../data/good_maps/wgs842merc_reprojection.xml')
        merc_bounds = mapnik.Box2d(-20037508.34, -
                                   20037508.34, 20037508.34, 20037508.34)
        m.maximum_extent = merc_bounds
        m.zoom_all()
        # note - fixAspectRatio is being called, then re-clipping to maxextent
        # which makes this hard to predict
        # eq_(m.envelope(),merc_bounds)

        #m = mapnik.Map(512,512)
        # mapnik.load_map(m,'../data/good_maps/wgs842merc_reprojection.xml')
        #merc_bounds = mapnik.Box2d(-20037508.34,-20037508.34,20037508.34,20037508.34)
        # m.zoom_to_box(merc_bounds)
        # eq_(m.envelope(),merc_bounds)

    def test_visual_zoom_all_rendering1():
        m = mapnik.Map(512, 512)
        mapnik.load_map(m, '../data/good_maps/wgs842merc_reprojection.xml')
        merc_bounds = mapnik.Box2d(-20037508.34, -
                                   20037508.34, 20037508.34, 20037508.34)
        m.maximum_extent = merc_bounds
        m.zoom_all()
        im = mapnik.Image(512, 512)
        mapnik.render(m, im)
        actual = '/tmp/mapnik-wgs842merc-reprojection-render.png'
        expected = 'images/support/mapnik-wgs842merc-reprojection-render.png'
        im.save(actual, 'png32')
        expected_im = mapnik.Image.open(expected)
        eq_(im.tostring('png32'),
            expected_im.tostring('png32'),
            'failed comparing actual (%s) and expected (%s)' % (actual,
                                                                'test/python_tests/' + expected))

    def test_visual_zoom_all_rendering2():
        m = mapnik.Map(512, 512)
        mapnik.load_map(m, '../data/good_maps/merc2wgs84_reprojection.xml')
        m.zoom_all()
        im = mapnik.Image(512, 512)
        mapnik.render(m, im)
        actual = '/tmp/mapnik-merc2wgs84-reprojection-render.png'
        expected = 'images/support/mapnik-merc2wgs84-reprojection-render.png'
        im.save(actual, 'png32')
        expected_im = mapnik.Image.open(expected)
        eq_(im.tostring('png32'),
            expected_im.tostring('png32'),
            'failed comparing actual (%s) and expected (%s)' % (actual,
                                                                'test/python_tests/' + expected))

    # maximum-extent read from map.xml
    def test_visual_zoom_all_rendering3():
        m = mapnik.Map(512, 512)
        mapnik.load_map(m, '../data/good_maps/bounds_clipping.xml')
        m.zoom_all()
        im = mapnik.Image(512, 512)
        mapnik.render(m, im)
        actual = '/tmp/mapnik-merc2merc-reprojection-render1.png'
        expected = 'images/support/mapnik-merc2merc-reprojection-render1.png'
        im.save(actual, 'png32')
        expected_im = mapnik.Image.open(expected)
        eq_(im.tostring('png32'),
            expected_im.tostring('png32'),
            'failed comparing actual (%s) and expected (%s)' % (actual,
                                                                'test/python_tests/' + expected))

    # no maximum-extent
    def test_visual_zoom_all_rendering4():
        m = mapnik.Map(512, 512)
        mapnik.load_map(m, '../data/good_maps/bounds_clipping.xml')
        m.maximum_extent = None
        m.zoom_all()
        im = mapnik.Image(512, 512)
        mapnik.render(m, im)
        actual = '/tmp/mapnik-merc2merc-reprojection-render2.png'
        expected = 'images/support/mapnik-merc2merc-reprojection-render2.png'
        im.save(actual, 'png32')
        expected_im = mapnik.Image.open(expected)
        eq_(im.tostring('png32'),
            expected_im.tostring('png32'),
            'failed comparing actual (%s) and expected (%s)' % (actual,
                                                                'test/python_tests/' + expected))

if __name__ == "__main__":
    setup()
    exit(run_all(eval(x) for x in dir() if x.startswith("test_")))
